###########################################################################
## (c) Maria Dascalu 2016
## Given g, c, n, k, generates g progressions using oneprog.py, and saves the
## Results of each progression to a result file in the format
## progresults_c#_k#_n#.txt
## in the Results folder.
###########################################################################

import oneprog
import time
import sys as Sys
from datetime import timedelta

###########################################################################
##############                  PROGRESS BAR                 ##############
###########################################################################

# @Vladimir Ignatyev's code modified by Greenstick on StackExchange
# http://stackoverflow.com/questions/3173320/text-progress-bar-in-the-console
# Further modified by me to serve my own nefarious purposes.
# Print iterations progress
def printProgress (iteration, total, start_time, prefix = '', suffix = '', decimals = 2, barLength = 20):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : number of decimals in percent complete (Int)
        barLength   - Optional  : character length of bar (Int)
    """
    filledLength    = int(round(barLength * iteration / float(total)))
    percents        = round(100.00 * (iteration / float(total)), decimals)
    bar             = '#' * filledLength + '-' * (barLength - filledLength)
    time_elapsed = str(timedelta(seconds=(time.time()-start_time)))
    Sys.stdout.write('Checked %s/%s. [%s] %s%s %s; time: %s\r' % (iteration, total, bar, percents, '%', suffix, time_elapsed)),
    Sys.stdout.flush()

###########################################################################
##############                 GATHERING DATA                ##############
###########################################################################

def add_0prog(prog_c, c, filename):
    """(dictionary) -> None"""
    directory = "0progs/" + filename
    fin = open(directory, 'a')
    fin.write("Progression:\n")
    for color in range(c-1):
        fin.write("COLOR {}: {} \n".format(color, prog_c[color]))
    fin.write("\n\n")
    fin.close()

def grabProgs(n, k, c, g, start_time):
    """(int, int, int, int, Time) -> list
    Takes all this information and generates g progressions given n, k, c.
    Saves to rfin, saves any 0 progs to filename. Also prints progress."""

    filename = "0progs_c{}_k{}_n{}.txt".format(c, k, n)
    rfin = "progresults_c{}_k{}_n{}.txt".format(c, k, n)
    countList = []
    count0 = 0
    printProgress(0, g, start_time)
    for i in range(1, g+1):
        count = oneprog.run(c, k, n)
        if count == 0:
            add_0prog(prog_c, c, filename)
            count0 += 1

        countList.append(count)
        saveTo(count, rfin)
        printProgress(i, g, start_time, suffix = "complete")

    if count0 > 0:
        print("\n Found {} 0-progs. Added to {}".format(count0, filename))

    return countList

def saveTo(result, filename):
    """Given a result and a place to save to, saves to that file."""
    directory = "Results/" + filename
    fin = open(directory, 'a')
    fin.write(str(result)+"\n")
    fin.close()


def run(g, n, k, c):
    print("\n")
    rfin = "progresults_c{}_k{}_n{}.txt".format(c, k, n)
    start_time = time.time()
    #print("Started at: {}".format(start_time))
    countList = grabProgs(n, k, c, g, start_time)
    print("\nSaved to {}".format(rfin))
    time_elapsed = timedelta(seconds= time.time() - start_time)
    print("\n--- %s time elapsed ---" % (time_elapsed))
    return countList, time_elapsed

#run(1, 100, 6, 2)
