###########################################################################
## (c) Maria Dascalu 2016
## Generates one progression, counts its monochromatic complete
## subprogressions of length k, and returns that number.
###########################################################################

import random

###########################################################################
##############             FIND K-SUBPROGRESSIONS            ##############
###########################################################################

def genNumbers(c, n):
    """(int, int) -> dict
    Given c and n, generates a color list where each number has a color and
    returns it.
    """
    progression = {}

    for i in range(1, n+1):
        progression[i] = random.randint(0, c-1)

    return progression

def byColor(prog):
    """(dict) -> dict
    Given a progression of numbers it reorganizes them so that all numbers of
    one color are grouped together.
    """
    prog_color = {}
    for key in prog:
        color = prog[key]
        if color not in prog_color:
            prog_color[color] = [key]
        else:
            prog_color[color].append(key)

    return prog_color
def countMono3(numbers):
    """(list) -> int
    Given a list of numbers (we know they're all the same color and sorted), counts number
    of monochromatic arithmetic progressions of length 3.
    """
    #numbers = sorted(numbers)
    big = numbers[-1]
    stop = len(numbers) - 2

    progList = []
    for a in numbers[:stop]:
        d = 1
        while (a+2*d) <= big:
            # we know it's not too high a d
            if (a+2*d) in numbers and (a+d) in numbers:
                mono = (a, a+d, a+2*d)
                progList.append(mono)
            d += 1
    return progList

def countMonok(numbers, k):
    """(list, int) -> int
    Given a list of numbers (we know they're all the same color), counts number
    of monochromatic arithmetic progressions of length k.
    """
    numbers = sorted(numbers)
    if k == 3:
        return countMono3(numbers)

    pfound = sorted(countMonok(numbers, k-1))
    found = []

    for prog in pfound:
        #print("Checking {}".format(prog))
        a = prog[0]
        d = prog[1] - prog[0]
        new = a + (k-1)*d
        #print("Is {} there?".format(new))
        if new in numbers:
            #print("yes")
            listprog = list(prog)
            listprog.append(new)
            newprog = tuple(listprog)
            #print("newprog = {}".format(newprog))
            found.append(newprog)

    return found

###########################################################################
##############            COUNTING FOR ONE PROG              ##############
###########################################################################

def run(c, k, n):
    prog = genNumbers(c, n)
    prog_color = byColor(prog)

    count = 0

    for color in range(c):
        try:
            numbers = prog_color[color]
        except:
            continue
        progList = countMonok(numbers, k)
        count += len(progList)

    return count

#run()
