#Load helper Libraries
library(Delaporte)

##Define Likelihood
DelaporteLL<- function(data,par){
  loglik <- -sum(ddelap(data,alpha=par[1],beta=par[2],lambda=par[3],log=TRUE))
  ###Penalties on negative parameter estimates
  loglik <-ifelse(par[1]<=0,Inf,loglik)
  loglik <-ifelse(par[2]<=0,Inf,loglik)
  loglik <-ifelse(par[3]<=0,Inf,loglik)
  return(loglik)
}

  MleTable<- c()


  ##replace with file location
  fn<-"results14.txt"
  data <- read.table(fn, header=TRUE,sep=" ")
  ###Translate to a vector of observations if necessary
  mydata<- c()
  for(i in 1:nrow(data)){
    mydata<-c(mydata,rep(data[i,1],data[i,2]))
  }
  #######################################################################
  #######################################################################
  ########################## MLE ########################################
  #######################################################################
  #######################################################################
  ##calculate the empirical pmf
  EMPpmf<-table(mydata)/length(mydata)
  
  #support based on the dataset for graphs later
  xseq<-seq(min(mydata),max(mydata),1)
  
  ##Gives reasonable starting values for parameters (MOM)
  starts<-MoMdelap(mydata) ##alpha, beta, lambda
  ##Find MLEs by optimizing the likelihood
  MLEs<-optim(par=starts,fn=DelaporteLL, data=mydata)$par
  DalphaMLE<-MLEs[1]
  DbetaMLE<-MLEs[2]
  DlambdaMLE<-MLEs[3]
  
  ##Get the pmf estimate using the MLEs
  DMLEpmf<-ddelap(xseq, DalphaMLE,DbetaMLE,DlambdaMLE, log = FALSE)
  
  #plug holes in PMF with 0 mass
  empPMFall<-rep(0,length(xseq))
  for(i in 1:length(xseq)){
    index<-which(names(EMPpmf)==xseq[i])
    if(length(index)!=0){
      empPMFall[i]=EMPpmf[which(names(EMPpmf)==xseq[i])]
    }
  } 
  
  #function for finding L1 distance
  L1.p<-function(){
    alphaf<-DalphaMLE
    betaf<-DbetaMLE
    lambdaf<-DlambdaMLE
    Dpmf<-ddelap(xseq,alphaf,betaf,lambdaf,log=FALSE)
    
    Dtailprob<-1-sum(ddelap(xseq, alphaf,betaf,lambdaf))
    L1=sum(abs(Dpmf-empPMFall))+Dtailprob
    return(L1)
  }
  
  #MLE Results row
  MleTable<-c(DalphaMLE,DbetaMLE,DlambdaMLE,L1.p());
  ##########PLOT
  maxy=max(max(DMLEpmf),max(EMPpmf))
  miny=max(min(DMLEpmf),max(EMPpmf))
  onep=.01*(maxy-miny)
  plot(cbind(as.integer(names(EMPpmf)),EMPpmf),type="h",ylim=c(0,maxy+onep),main="Delaporte")
  points(xseq,DMLEpmf,pch=1,col="red")
  legend("topright", c("Delaporte w/ MLE"), pch = c(1), col=c("red"), inset = .02)

##Print results
names(MleTable)<-c("alpha", "beta", "lambda","L1 dist.")
MleTable
