library(Delaporte)

MoMdelap2<-function(dat){
  m=mean(dat)
  v=var(dat)
  sk=skewness(dat)
  f<-sqrt(sk^2 * v^3 * (v-m)^2)
  
  num<-2*m^2*v+2*v*f - m*(2*f+v^2*(sk^2*v + 7)) + 6*v^3
  denom<- 4*m^2 - 12*m*v + v^2*(9-sk^2*v);
  lambda<-num/denom

  num<- -4*(m-v)^4
  denom<- 4*m^3-16*m^2*v - 4*m*f+6*v*f+m*v^2*(sk^2*v+21)-v^3*(sk^2*v+9)
  alpha=num/denom  

  num<- -2*m^2 + f + 5*m*v - 3*v^2 
  denom<- 2*(m-v)^2
  beta=num/denom
  
  if(beta>0 && lambda>0 && alpha>0){
    c(alpha, beta, lambda)
  }else{
    stop("data provided inapproriate negative mom estimates")
  }
}

options(java.parameters = "-xmx2500m")

DelaporteLL<- function(data,par){
  loglik <- -sum(ddelap(data,alpha=par[1],beta=par[2],lambda=par[3],log=TRUE))
  ###Penalties on negative parameter estimates
  loglik <-ifelse(par[1]<=0,Inf,loglik)
  loglik <-ifelse(par[2]<=0,Inf,loglik)
  loglik <-ifelse(par[3]<=0,Inf,loglik)
  return(loglik)
}

simCheck<-function(n,alpha,beta,lambda,sims){
  options(show.error.messages = FALSE)
  aMOMs<-c()
  bMOMs<-c()
  lMOMs<-c()
  aMLEs<-c()
  bMLEs<-c()
  lMLEs<-c()
  for(i in 1:sims){
    if(i%%10==0){
      print(paste("...running simulation ",i,sep=""));
      flush.console();
    }
    data<-rdelap(n=n,alpha = alpha,beta = beta,lambda = lambda)
    #find method of moments estimators
    moms<-try(MoMdelap2(data))
    if(length(moms)==3){
      aMOMs<-c(aMOMs,moms[1])
      bMOMs<-c(bMOMs,moms[2])
      lMOMs<-c(lMOMs,moms[3])
    }else{
      i=i-1;
      next;
    }
    #find maximumlikelihood estimators
    mles<-optim(par=moms,fn=DelaporteLL, data=data)$par
    aMLEs<-c(aMLEs,mles[1])
    bMLEs<-c(bMLEs,mles[2])
    lMLEs<-c(lMLEs,mles[3])
  }
  #create output table
  output<-list(aMOM=c(mean(aMOMs),sd(aMOMs)),
               bMOMs=c(mean(bMOMs),sd(bMOMs)),
               lMOMs=c(mean(lMOMs),sd(lMOMs)),
               aMLEs=c(mean(aMLEs),sd(aMLEs)),
               bMLEs=c(mean(bMLEs),sd(bMLEs)),
               lMLEs=c(mean(lMLEs),sd(lMLEs)))
  return(output)
}


out1<-simCheck(n=100,alpha=1.84,beta=7.89,lambda=16.75,100)
out1_1k<-simCheck(n=1000,alpha=1.84,beta=7.89,lambda=16.75,100)
out1_10k<-simCheck(n=10000,alpha=1.84,beta=7.89,lambda=16.75,100)
out1_100k<-simCheck(n=100000,alpha=1.84,beta=7.89,lambda=16.75,100)


out2<-simCheck(n=100,alpha=3.74,beta=15.46,lambda=93.57,100)
out2_1k<-simCheck(n=1000,alpha=3.74,beta=15.46,lambda=93.57,100)
out2_10k<-simCheck(n=10000,alpha=3.74,beta=15.46,lambda=93.57,100)
out2_100k<-simCheck(n=100000,alpha=3.74,beta=15.46,lambda=93.57,100)

out3<-simCheck(n=100,alpha=9.45,beta=163.51,lambda=2178.57,100)
out3_1k<-simCheck(n=1000,alpha=9.45,beta=163.51,lambda=2178.57,100)
out3_10k<-simCheck(n=10000,alpha=9.45,beta=163.51,lambda=2178.57,100)
out3_100k<-simCheck(n=100000,alpha=9.45,beta=163.51,lambda=2178.57,100)