###########################################################################
## (c) Maria Dascalu 2016
## Given g, c, n, k, generates g graphs using onegraph.py, and saves the Results
## of each graph to a result file in the format
## results_c#_k#_n#.txt
## in the Results folder.
###########################################################################

import onegraph
import populate
import time
import sys as Sys
from datetime import timedelta

###########################################################################
##############                  PROGRESS BAR                 ##############
###########################################################################

# @Vladimir Ignatyev's code modified by Greenstick on StackExchange
# http://stackoverflow.com/questions/3173320/text-progress-bar-in-the-console
# Further modified by me to serve my own nefarious purposes.
# Print iterations progress
def printProgress (iteration, total, start_time, prefix = '', suffix = '', decimals = 2, barLength = 20):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : number of decimals in percent complete (Int)
        barLength   - Optional  : character length of bar (Int)
    """
    filledLength    = int(round(barLength * iteration / float(total)))
    percents        = round(100.00 * (iteration / float(total)), decimals)
    bar             = '#' * filledLength + '-' * (barLength - filledLength)
    time_elapsed = str(timedelta(seconds=(time.time()-start_time)))
    Sys.stdout.write('Checked %s/%s. [%s] %s%s %s; time: %s\r' % (iteration, total, bar, percents, '%', suffix, time_elapsed)),
    Sys.stdout.flush()

###########################################################################
##############                 GATHERING DATA                ##############
###########################################################################

def add_0graph(graph_c, c, filename, cat, delete):
    """(dictionary) -> None
    If a 0graph is found, the necessary information is added to the
    relevant text file.
    """
    directory = "0graphs/" + filename
    fin = open(directory, 'a')
    fin.write("Graph-{} (ignore vertices {}):\n".format(cat, delete))
    for color in range(c-1):
        fin.write("COLOR {}:\n".format(color))
        for i in range(len(graph_c)):
            try:
                vertices = graph_c[i][color]
            except:
                continue
            fin.write("{}: {}\n".format(i, graph_c[i][color]))
    fin.write("\n\n")
    fin.close()

def grabGraphs(n, k, c, g, cat, start_time):
    """(int, int, int, int, string, Time) -> list
    Takes all this information and generates g graphs given n, k, c, cat.
    Saves to rfin, saves any 0 graphs to filename. Also prints progress.
    """

    filename = "0graphs_c{}_k{}_n{}.txt".format(c, k, n)
    rfin = "{}esults_c{}_k{}_n{}.txt".format(cat, c, k, n)
    countList = []
    count0 = 0
    printProgress(0, g, start_time)
    for i in range(1, g+1):
        delete = []
        if cat == "r":
            graph = populate.populate(n, c)
        elif cat == "h":
            graph = populate.populatehalf(n)
        else:
            if i % 10000 == 1:
                graph = populate.generate51()
                #print("New graph!")
            delete = populate.choose8()
        #print(delete)
        graph_c = populate.vbycolor(graph)
        count = onegraph.howmany(graph_c, c, k, delete)
        if count == 0:
            add_0graph(graph_c, c, filename, cat, delete)
            count0 += 1

        countList.append(count)
        saveTo(count, rfin)
        printProgress(i, g, start_time, suffix = "complete")

    if count0 > 0:
        print("\n Found {} 0-graphs. Added to {}".format(count0, filename))

    return countList

def saveTo(result, filename):
    """Given a result and a place to save to, saves to that file."""
    directory = "Results/" + filename
    fin = open(directory, 'a')
    fin.write(str(result)+"\n")
    fin.close()


def run(g, n, k, c, cat):
    print("\n")
    rfin = "{}esults_c{}_k{}_n{}.txt".format(cat, c, k, n)
    start_time = time.time()
    #print("Started at: {}".format(start_time))
    countList = grabGraphs(n, k, c, g, cat, start_time)
    print("\nSaved to {}".format(rfin))
    time_elapsed = timedelta(seconds= time.time() - start_time)
    print("\n--- %s time elapsed ---" % (time_elapsed))
    return countList, time_elapsed

#run(1, 100, 6, 2)
