###########################################################################
## (c) Maria Dascalu 2016
## Generates one graph, counts its monochromatic complete subgraphs on k
## vertices, and returns that number.
###########################################################################

#import time
import random



###########################################################################
##############                FIND K-SUBGRAPHS               ##############
###########################################################################

def intersection(first, *others):
    """(list, *list) -> set
    Returns intersection of arbitrarily many lists."""
    return set(first).intersection(*others)

def find3(colorgraph, r, ignore):
    """(dictionary, list, int)-> list
    Returns all found 3-subgraphs."""
    vertices = sorted(colorgraph.keys())
    n = len(colorgraph)
    found_subgraphs = []

    for va in vertices:
        if va in ignore:
            continue
        if va > n-2:
            break
        try:
            va_links = colorgraph[va][r]
        except:
            continue
        if len(va_links) < 2:
            continue
        for vb in va_links:
            if vb < va or vb not in vertices:
                continue
            try:
                vb_links = colorgraph[vb][r]
            except:
                continue
            common = intersection(va_links, vb_links)
            if len(common) == 0:
                continue
            for vc in common:
                if vc < va or vc < vb:
                    continue
                found_subgraphs.append((va, vb, vc))
    return found_subgraphs

def findk(colorgraph, r, k, ignore):
    """(dictionary, list, int, int) -> list
    Returns the list of all found k-subgraphs."""
    n = len(colorgraph)
    if k == 3:
        return find3(colorgraph, r, ignore)

    prefound = sorted(findk(colorgraph, r, k-1, ignore))
    found = []
    npf = len(prefound)

    for i in range(k, n+1):
        if i in ignore:
            continue
        try:
            ilink = colorgraph[i][r]
        except:
            continue
        if len(ilink) < k-1:
            continue
        for subgraph in prefound:
            #print("{} vs. {}:{}".format(i, subgraph, max(max(subgraph), i)), end=" ")
            if i <= max(subgraph):
                #print("\b Skipping.")
                continue
            #print("\b Testing.")
            flag = True
            for number in subgraph:
                try:
                    nlink = colorgraph[number][r]
                    #print('Testing {}: {}'.format(number, nlink))
                except:
                    break

                if i not in nlink:
                    #print("Doesn't match up.")
                    flag = False
                    break
            if flag:
                kgraph = list(subgraph)
                kgraph.append(i)
                ktuple = tuple(kgraph)
                found.append(ktuple)
                #print("Adding {}".format(ktuple))

    return found

###########################################################################
##############            COUNTING FOR ONE GRAPH             ##############
###########################################################################

def howmany(graph_c, c, k, ignore):
    """(dictionary, int, int) -> int
    Takes a graph organized by color, the number of colors, and k, and
    returns the number of found_ksubgraphs."""
    found_ksubgraphs = []
    for color in range(c):
        #print("COLOR {}".format(color))
        found = findk(graph_c, color, k, ignore)
        found_ksubgraphs.extend(found)
    #print("\n\nFound: ")
    #print(sorted(found_ksubgraphs))
    #print("{}: {}".format(k, len(found_ksubgraphs)))
    return len(found_ksubgraphs)

#
# def main():
#     n = 4
#     k = 3
#     c = 2
#     start_time = time.time()
#     # graph. Will contain all the keys and their links, as well as colors.
#     graph = populate.populate(n, c)
#     graph_c = vbycolor(graph)
#     print(graph_c)
#
#     # print("Graph by color: {} \n".format(graph_c))
#     count = howmany(graph_c, c, k)
#     print("Count {}".format(count))
#
#     print("--- %s seconds ---" % (time.time() - start_time))
#
# main()
